@@ -13,21 +13,41 @@
 .LEGACY_INSTALL_CMD <-
     "source(\"https://bioconductor.org/biocLite.R\")"
 
-.VERSION_SENTINEL <- local({
-    version <- package_version(list())
-    class(version) <- c("unknown_version", class(version))
-    version
-})
-
 .VERSION_MAP_SENTINEL <- data.frame(
-    Bioc = .VERSION_SENTINEL,
-    R = .VERSION_SENTINEL,
+    Bioc = package_version(list()),
+    R = package_version(list()),
     BiocStatus = factor(
         factor(),
         levels = c("out-of-date", "release", "devel", "future")
-    )
+    ),
+    RPSM = character(),
+    MRAN = character()
 )
 
+.version_sentinel <-
+    function(msg)
+{
+    version <- package_version(list())
+    structure(
+        unclass(version),
+        msg = msg,
+        class = c("version_sentinel", class(version))
+    )
+}
+
+.version_sentinel_msg <-
+    function(x)
+{
+    attr(x, "msg")
+}
+
+#' @export
+format.version_sentinel <-
+    function(x, ...)
+{
+    paste0("unknown version: ", .version_sentinel_msg(x))
+}
+
 .version_compare <-
     function(v1, v2)
 {
@@ -69,6 +89,23 @@
     txt
 }
 
+.version_map_config_element <-
+    function(txt, tag)
+{
+    grps <- grep("^[^[:blank:]]", txt)
+    start <- match(grep(tag, txt), grps)
+    if (!length(start))
+        return(setNames(character(), character()))
+    end <- ifelse(length(grps) < start + 1L, length(txt), grps[start + 1] - 1L)
+    map <- txt[seq(grps[start] + 1, end)]
+    map <- trimws(gsub("\"", "", sub(" #.*", "", map)))
+
+    pattern <- "(.*): (.*)"
+    key <- sub(pattern, "\\1", map)
+    value <- sub(pattern, "\\2", map)
+    setNames(value, key)
+}
+
 .version_map_get_online <-
     function(config)
 {
@@ -86,14 +123,12 @@
     if (inherits(txt, "error"))
         return(.VERSION_MAP_SENTINEL)
 
-    grps <- grep("^[^[:blank:]]", txt)
-    start <- match(grep("r_ver_for_bioc_ver", txt), grps)
-    map <- txt[seq(grps[start] + 1, grps[start + 1] - 1)]
-    map <- trimws(gsub("\"", "", sub(" #.*", "", map)))
+    bioc_r_map <- .version_map_config_element(txt, "r_ver_for_bioc_ver")
+    bioc <- package_version(names(bioc_r_map))
+    r <- package_version(unname(bioc_r_map))
 
-    pattern <- "(.*): (.*)"
-    bioc <- package_version(sub(pattern, "\\1", map))
-    r <- package_version(sub(pattern, "\\2", map))
+    bioc_rspm_map <- .version_map_config_element(txt, "rspm_ver_for_bioc_ver")
+    bioc_mran_map <- .version_map_config_element(txt, "mran_ver_for_bioc_ver")
 
     pattern <- "^release_version: \"(.*)\""
     release <- package_version(
@@ -125,8 +160,10 @@
         BiocStatus = factor(
             status,
             levels = c("out-of-date", "release", "devel", "future")
-        )
-    ))
+        ),
+        RSPM = unname(bioc_rspm_map[as.character(bioc)]),
+        MRAN = unname(bioc_mran_map[as.character(bioc)])
+   ))
 }
 
 .version_map_get_offline <-
@@ -146,7 +183,9 @@
         BiocStatus = factor(
             NA,
             levels = c("out-of-date", "release", "devel", "future")
-        )
+        ),
+        RSPM = NA_character_,
+        MRAN = NA_character_
     ))
 }
 
@@ -156,7 +195,7 @@
     if (!.version_validity_online_check())
         .version_map_get_offline()
     else {
-        if (is.null(config))
+        if (is.null(config) || !nchar(config))
             config <- "https://bioconductor.org/config.yaml"
         .version_map_get_online(config)
     }
@@ -165,28 +204,43 @@
 .version_map <- local({
     version_map <- .VERSION_MAP_SENTINEL
     function() {
+        config <- Sys.getenv("BIOCONDUCTOR_CONFIG_FILE")
+        config <- getOption("BIOCONDUCTOR_CONFIG_FILE", config)
         if (identical(version_map, .VERSION_MAP_SENTINEL))
-            version_map <<- .version_map_get()
+            version_map <<- .version_map_get(config)
         version_map
     }
 })
 
+.version_field <-
+    function(field)
+{
+    map <- .version_map()
+    if (identical(map, .VERSION_MAP_SENTINEL))
+        return(NA)
+    idx <- match(version(), map[["Bioc"]])
+    map[idx, field]
+}
+
+.get_R_version <- function()
+    getRversion()
+
+
 .version_validity <-
-    function(version)
+    function(version, map = .version_map(), r_version = .get_R_version())
 {
     if (identical(version, "devel"))
         version <- .version_bioc("devel")
     version <- .package_version(version)
 
-    if (identical(version, .VERSION_SENTINEL))
-        return(.VERSION_MAP_UNABLE_TO_VALIDATE)
+    if (inherits(version, "version_sentinel"))
+        return(.version_sentinel_msg(version))
 
     if (version[, 1:2] != version)
         return(sprintf(
             "version '%s' must have two components, e.g., '3.7'", version
         ))
 
-    map <- .version_map()
     if (identical(map, .VERSION_MAP_SENTINEL))
         return(.VERSION_MAP_UNABLE_TO_VALIDATE)
 
@@ -195,11 +249,23 @@
             "unknown Bioconductor version '%s'; %s", version, .VERSION_HELP
         ))
 
-    required <- map$R[map$Bioc == version]
-    if (!getRversion()[, 1:2] %in% required) {
-        return(sprintf(
-            "Bioconductor version '%s' requires R version '%s'; %s",
-            version, head(required, 1), .VERSION_HELP
+    required <- map$R[map$Bioc == version & map$BiocStatus != "future"]
+    r_version <- r_version[, 1:2]
+    if (!r_version %in% required) {
+        rec <- map[map$R == r_version, , drop = FALSE]
+        rec_fun <- ifelse("devel" %in% rec$BiocStatus, head, tail)
+
+        rec_msg <- if ("future" %in% rec$BiocStatus)
+            "R version is too new"
+        else
+            sprintf(
+                "use `BiocManager::install(version = '%s')` with R version %s",
+                rec_fun(rec$Bioc, 1), r_version
+            )
+
+        return(.msg(
+            "Bioconductor version '%s' requires R version '%s'; %s; %s",
+            version, head(required, 1), rec_msg, .VERSION_HELP
         ))
     }
 
@@ -272,7 +338,7 @@
 {
     map <- .version_map()
     if (identical(map, .VERSION_MAP_SENTINEL))
-        return(.VERSION_MAP_UNABLE_TO_VALIDATE)
+        return(.version_sentinel(.VERSION_MAP_UNABLE_TO_VALIDATE))
 
     map <- map[map$R == getRversion()[, 1:2],]
     if ("release" %in% map$BiocStatus)
@@ -316,7 +382,7 @@
     tryCatch({
         packageVersion("BiocVersion")[, 1:2]
     }, error = function(e) {
-        .VERSION_SENTINEL
+        .version_sentinel("package 'BiocVersion' not installed")
     })
 }
 
@@ -367,8 +433,8 @@ version <-
 #'
 #' @md
 #' @export
-print.unknown_version <-
+print.version_sentinel <-
     function(x, ...)
 {
-    cat("'unknown'\n")
+    cat(format(x), "\n")
 }
